!******************************************************************************!
!                                                                              !
!    collier_interface_rcl.f90                                                 !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016,2017 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!

module collier_interface_rcl

!------------------------------------------------------------------------------!

  use globals_rcl
  use collier, only :           &
      Init_cll,                 &
      InitCacheSystem_cll,      &
      SetMode_cll,              &
      SwitchOnTenRed_cll,       &
      SwitchOffTenRed_cll,      &
      SetCacheLevel_cll,        &
      SwitchOffCache_cll,       &
      SwitchOffCacheSystem_cll, &
      SetCacheMode_cll,         &
      setminf2_cll,             &
      SetDeltaUV_cll,           &
      SetDeltaIR_cll,           &
      SetMuIR2_cll,             &
      SetMuUV2_cll,             &
      InitEvent_cll,            &
      TNten_cll,                &
      GetAccFlag_cll,           &
      SetCritAcc_cll,           &
      GetCritAcc_cll,           &
      SetReqAcc_cll,            &
      GetReqAcc_cll,            &
      SetErrStop_cll,           &
      GetErrFlag_cll,           &
      SetAccuracy_cll

  use modelfile, only: get_n_particles_mdl, get_particle_cmass2_reg_mdl, &
                       is_particle_mdl, get_particle_mass_reg_mdl
  use input_rcl, only: get_delta_uv_rcl, get_delta_ir_rcl, &
                       get_mu_uv_rcl, get_mu_ir_rcl

!------------------------------------------------------------------------------!

 implicit none

!------------------------------------------------------------------------------!

 contains

!------------------------------------------------------------------------------!
  
  subroutine initialize_collier

  integer                  :: pr,nPropMax,rankMax,i,n,n_particles
  integer, allocatable     :: nProp(:), rank(:)
  complex(dp), allocatable :: m2(:)
  real(dp) :: d,d2,muUV,muIR

  allocate (nProp(prTot)); nProp = 0
  allocate (rank(prTot)); rank = 0
  do pr = 1,prTot
    if (loop(pr)) nProp(pr) = maxval(legsti(:tiTot(pr),pr))
    if (loop(pr)) rank(pr) = maxval(rankti(0:tiTot(pr),pr))
  enddo
  nPropMax = maxval(nProp)
  rankMax = maxval(rank)

  if (nPropMax .lt. 4) then
    call openOutput

    write(nx,*)
    write(nx,'(1x,75("-"))')
    write(nx, *) ' Initialising collier with 4 legs.'
    write(nx,'(1x,75("-"))')
    if (trim(collier_output_dir) .eq. 'default') then
      call Init_cll(4,noreset=.true.)
    else
      call Init_cll(nPropMax,                             &
                    folder_name=trim(collier_output_dir), &
                    noreset=.true.)
    endif
  else
    call openOutput
    write (nx,'(a,i2,a,i2,a)') ' Initialising collier with', nPropMax, &
                             ' legs and rank', rankMax, '.'
    write(nx,'(1x,75("-"))')
    if (trim(collier_output_dir) .eq. 'default') then
      call Init_cll(nPropMax,rmax=rankMax,noreset=.true.)
    else
      call Init_cll(nPropMax,rmax=rankMax,                &
                    folder_name=trim(collier_output_dir), &
                    noreset=.true.)
    endif
  end if

  if (.not.allocated(nCache)) then
    allocate(nCache(1:prTot)); nCache = 1
  endif

  if (.not.allocated(cacheOn)) then
    allocate(cacheOn(1:prTot)); cacheOn = .true.
  endif

  allocate(nCacheTot(0:prTot),tiCache(prTot))

  nCacheTot = 0
  do pr = 1,prTot
    if (loop(pr)) tiCache(pr) = ceiling(tiTot(pr)*1d0/nCache(pr))
    nCacheTot(pr) = nCacheTot(pr-1) + nCache(pr)
  enddo

  call InitCacheSystem_cll(nCacheTot(prTot),nPropMax)

  call SetMode_cll(1)

  call SwitchOnTenRed_cll

  do pr = 1,prTot
    call SetCacheLevel_cll(pr,nProp(pr))
  enddo

  do pr = 1,prTot
    do n = nCacheTot(pr-1)+1,nCacheTot(pr)
      call SetCacheLevel_cll(n,nProp(pr))
      if (.not.cacheOn(pr)) call SwitchOffCache_cll(n)
    enddo
  enddo

  if (ifail.ge.0) call SetErrStop_cll(-11) 

  n = 0
  n_particles = get_n_particles_mdl()
  do i = 1,n_particles
    if (get_particle_mass_reg_mdl(i) .eq. 2 .and. is_particle_mdl(i))  n = n + 1
  end do

  allocate (m2(n))

  n = 0
  do i = 1,n_particles
    if (get_particle_mass_reg_mdl(i) .eq. 2 .and. is_particle_mdl(i))  then
      n = n + 1
      m2(n) = get_particle_cmass2_reg_mdl(i)
    endif
  enddo
  call setminf2_cll (n,m2)
  deallocate (m2)

  ! Fix for current modelfiles. Needs to be set to deltaUV when problems are
  ! resolved
  call SetDeltaUV_cll (0d0)
  call get_delta_ir_rcl(d,d2)
  call SetDeltaIR_cll (d,d2)

  call get_mu_ir_rcl(muIR)
  call SetMuIR2_cll(muIR**2)
  call get_mu_uv_rcl(muUV)
  call SetMuUV2_cll(muUV**2) 

  end subroutine initialize_collier

!------------------------------------------------------------------------------!

end module collier_interface_rcl

!------------------------------------------------------------------------------!
